package net.cattaka.rdbassistant.gui.connectioninfo;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JPanel;

import net.cattaka.rdbassistant.config.RdbaConnectionInfo;
import net.cattaka.rdbassistant.dummy.DummyRdbaConnectionInfo;
import net.cattaka.rdbassistant.mysql.MySqlRdbaConnectionInfo;
import net.cattaka.rdbassistant.oracle.OracleRdbaConnectionInfo;
import net.cattaka.util.ExceptionHandler;
import net.cattaka.util.MessageBundle;

public class RdbaConnectionInfoEditorDialog extends JDialog {
	private static final long serialVersionUID = 1L;
	private static String[] rdbmsListItem = {"Oracle","MySql","Dummy"};
	private static Class<?>[] RDBA_CONNECTION_INFO_CLASSES = {
		OracleRdbaConnectionInfo.class,
		MySqlRdbaConnectionInfo.class,
		DummyRdbaConnectionInfo.class,
	}; 
	private RdbaConnectionInfo rdbaConnectionInfo = null;
	
	private JComboBox rdbmsComboBox;
	private JPanel editorPanel;
	private RdbaConnectionInfoEditor rdbaConnectionInfoEditor;

	// ========================================================================
	
	class ActionListenerForEditorFrame implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getActionCommand().equals("ok")) {
				rdbaConnectionInfo = rdbaConnectionInfoEditor.save();
				if (rdbaConnectionInfo != null) {
					setVisible(false);
				}
			} else if (e.getActionCommand().equals("cancel")) {
				rdbaConnectionInfo = null;
				setVisible(false);
			}
		}
	}
	
	class ItemListenerForEditorPane implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			updateRdbaConnectionInfo();
		}
	}
	
	// ========================================================================
	
	public RdbaConnectionInfoEditorDialog(Frame parentFrame) {
		super(parentFrame);
		this.setSize(300,200);
		this.setModal(true);
		makeLayout();
		updateRdbaConnectionInfo();
	}
	
	private void makeLayout() {
		ActionListener al = new ActionListenerForEditorFrame();
		JButton okButton = new JButton();
		JButton cancelButton = new JButton();
		okButton.setText(MessageBundle.getMessage("ok"));
		okButton.setActionCommand("ok");
		okButton.addActionListener(al);
		cancelButton.setText(MessageBundle.getMessage("cancel"));
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(al);
		
		rdbmsComboBox = new JComboBox();
		rdbmsComboBox.addItemListener(new ItemListenerForEditorPane());
		editorPanel = new JPanel();
		editorPanel.setLayout(new GridLayout());
		
		for (int i=0;i<rdbmsListItem.length;i++) {
			rdbmsComboBox.addItem(rdbmsListItem[i]);
		}
		
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbc.gridheight = 1;
		gbc.weightx=0.0;
		gbc.weighty=0.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbl.setConstraints(rdbmsComboBox, gbc);

		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx=1.0;
		gbc.weighty=1.0;
		gbc.gridy++;
		gbl.setConstraints(editorPanel, gbc);
		
		gbc.gridwidth = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx=0.0;
		gbc.weighty=0.0;
		gbc.gridy++;
		gbl.setConstraints(okButton, gbc);
		gbc.gridx++;
		gbl.setConstraints(cancelButton, gbc);
		
		this.getContentPane().setLayout(gbl);
		this.getContentPane().add(rdbmsComboBox);
		this.getContentPane().add(editorPanel);
		this.getContentPane().add(okButton);
		this.getContentPane().add(cancelButton);
	}

	@Override
	public void setVisible(boolean b) {
		super.setVisible(b);
	}
	
	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING ) {
			this.rdbaConnectionInfo = null;
			this.setVisible(false);
	    }
	}
	
	public RdbaConnectionInfo getRdbaConnectionInfo() {
		return rdbaConnectionInfo;
	}

	public void setRdbaConnectionInfo(RdbaConnectionInfo rdbaConnectionInfo) {
		this.rdbaConnectionInfo = rdbaConnectionInfo;
		this.updateEditorPanel();
	}

	private void updateRdbaConnectionInfo() {
		int idx = rdbmsComboBox.getSelectedIndex();
		try {
			rdbaConnectionInfo = (RdbaConnectionInfo)RDBA_CONNECTION_INFO_CLASSES[idx].newInstance();
		} catch (Exception exc) {
			ExceptionHandler.error(exc);
			throw new RuntimeException(exc);
		}
		updateEditorPanel();
	}
	
	private void updateEditorPanel() {
		try {
			rdbaConnectionInfoEditor = rdbaConnectionInfo.createEditor();
			rdbaConnectionInfoEditor.load(rdbaConnectionInfo);
			editorPanel.removeAll();
			editorPanel.add(rdbaConnectionInfoEditor);
			editorPanel.validate();
		} catch(Exception e) {
			throw new RuntimeException(e);
		}
	}
}
