package net.cattaka.rdbassistant.gui.sql;

import javax.swing.text.BadLocationException;

import net.cattaka.swing.JPopupMenuForStandardText;
import net.cattaka.swing.text.StdStyledDocument;
import net.cattaka.swing.text.StdTextPane;
import net.cattaka.swing.text.TextLineInfo;
import net.cattaka.util.ExceptionHandler;

public class SqlTextPanel extends StdTextPane {
	private static final long serialVersionUID = 1L;

	public SqlTextPanel() {
		super();
	}

	public SqlTextPanel(JPopupMenuForStandardText popupMenu) {
		super(popupMenu);
	}
	
	public void doCommentOut(String comment) {
		StdStyledDocument ssd = getStdStyledDocument();
		int startPos = this.getSelectionStart();
		int endPos = this.getSelectionEnd();
		if (ssd == null || startPos >= ssd.getLength()) {
			return;
		}
		if (endPos > startPos) {
			endPos = endPos-1;
		}
		
		boolean recordUndoSepalartor = ssd.isRecordUndoSepalartor();
		ssd.addUndoSepalartor();
		ssd.setRecordUndoSepalartor(false);
		
		try {
			TextLineInfo startTli = ssd.getLine(startPos);
			if (startTli != null) {
				boolean removeFlag = true;
				{
					TextLineInfo nextTli = startTli;
					while(nextTli != null && nextTli.getStartPos() <= endPos) {
						if (nextTli.getLine().length() >= comment.length()
								&& nextTli.getLine().startsWith(comment)) {
							// コメントアウト済み
						} else {
							// コメントアウト未だ
							removeFlag = false;
							break;
						}			
						nextTli = ssd.getNextLine(nextTli);
					}
				}
				
				if (removeFlag) {
					TextLineInfo nextTli = startTli;
					while(nextTli != null && nextTli.getStartPos() <= endPos) {
						ssd.remove(nextTli.getStartPos(), comment.length());
						endPos -= comment.length();
						nextTli = ssd.getLine(nextTli.getEndPos() - comment.length() + 1);
					}
				} else {
					TextLineInfo nextTli = startTli;
					while(nextTli != null && nextTli.getStartPos() <= endPos) {
						ssd.insertString(nextTli.getStartPos(), comment, null);
						endPos += comment.length();
						nextTli = ssd.getLine(nextTli.getEndPos() + comment.length() + 1);
					}
				}
			}
		} catch (BadLocationException e) {
			ExceptionHandler.error(e);
		} finally {
			ssd.setRecordUndoSepalartor(recordUndoSepalartor);
			ssd.addUndoSepalartor();
		}
	}
}
